@@ -15,9 +15,9 @@
 
 #include "tls/s2n_security_policies.h"
 
+#include "crypto/s2n_pq.h"
 #include "crypto/s2n_rsa_pss.h"
 #include "crypto/s2n_rsa_signing.h"
-#include "pq-crypto/s2n_pq.h"
 #include "s2n_test.h"
 #include "testlib/s2n_testlib.h"
 #include "tls/s2n_kem.h"
@@ -58,8 +58,6 @@ int main(int argc, char **argv)
     BEGIN_TEST();
     EXPECT_SUCCESS(s2n_disable_tls13_in_test());
 
-    EXPECT_TRUE(s2n_array_len(ALL_SUPPORTED_KEM_GROUPS) == S2N_SUPPORTED_KEM_GROUPS_COUNT);
-
     DEFER_CLEANUP(struct s2n_cert_chain_and_key *rsa_chain_and_key = NULL, s2n_cert_chain_and_key_ptr_free);
     EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&rsa_chain_and_key,
             S2N_DEFAULT_TEST_CERT_CHAIN, S2N_DEFAULT_TEST_PRIVATE_KEY));
@@ -81,15 +79,15 @@ int main(int argc, char **argv)
         /* TLS 1.3 + PQ checks */
         if (security_policy->kem_preferences->tls13_kem_group_count > 0) {
             /* Ensure that no TLS 1.3 KEM group preference lists go over max supported limit */
-            EXPECT_TRUE(security_policy->kem_preferences->tls13_kem_group_count <= S2N_SUPPORTED_KEM_GROUPS_COUNT);
+            EXPECT_TRUE(security_policy->kem_preferences->tls13_kem_group_count <= S2N_KEM_GROUPS_COUNT);
 
             /* Ensure all TLS 1.3 KEM groups in all policies are in the global list of all supported KEM groups */
             for (size_t i = 0; i < security_policy->kem_preferences->tls13_kem_group_count; i++) {
                 const struct s2n_kem_group *kem_group = security_policy->kem_preferences->tls13_kem_groups[i];
 
                 bool kem_group_is_supported = false;
-                for (size_t j = 0; j < S2N_SUPPORTED_KEM_GROUPS_COUNT; j++) {
-                    if (kem_group->iana_id == ALL_SUPPORTED_KEM_GROUPS[j]->iana_id) {
+                for (size_t j = 0; j < kem_preferences_all.tls13_kem_group_count; j++) {
+                    if (kem_group->iana_id == kem_preferences_all.tls13_kem_groups[j]->iana_id) {
                         kem_group_is_supported = true;
                         break;
                     }
@@ -108,126 +106,6 @@ int main(int argc, char **argv)
         }
 
         if (has_tls_13_cipher) {
-            /* Validate that s2n_tls13_default_sig_scheme() is successful on all TLS 1.3 Security Policies for all
-             * TLS 1.3 Ciphers */
-            {
-                struct s2n_cipher_suite tls_13_ciphers[] = {
-                    s2n_tls13_aes_128_gcm_sha256,
-                    s2n_tls13_aes_256_gcm_sha384,
-                    s2n_tls13_chacha20_poly1305_sha256
-                };
-
-                for (size_t i = 0; i < s2n_array_len(tls_13_ciphers); i++) {
-                    struct s2n_config *config = s2n_config_new();
-                    EXPECT_NOT_NULL(config);
-
-                    if (security_policy_selection[policy_index].security_policy->minimum_protocol_version > s2n_get_highest_fully_supported_tls_version()) {
-                        /* We purposefully do not allow users to configure Security Policies with a minimum allowed TLS
-                         * versions that are greater than what libcrypto supports. */
-                        EXPECT_FAILURE(s2n_config_set_cipher_preferences(config, security_policy_selection[policy_index].version));
-                        EXPECT_SUCCESS(s2n_config_free(config));
-                        continue;
-                    }
-
-                    struct s2n_cert_chain_and_key *default_cert;
-                    EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&default_cert, S2N_DEFAULT_TEST_CERT_CHAIN, S2N_DEFAULT_TEST_PRIVATE_KEY));
-
-                    EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(config, default_cert));
-                    EXPECT_TRUE(config->is_rsa_cert_configured);
-
-                    struct s2n_connection *client_conn = s2n_connection_new(S2N_CLIENT);
-                    struct s2n_connection *server_conn = s2n_connection_new(S2N_SERVER);
-                    EXPECT_NOT_NULL(client_conn);
-                    EXPECT_NOT_NULL(server_conn);
-
-                    EXPECT_SUCCESS(s2n_config_set_cipher_preferences(config, security_policy_selection[policy_index].version));
-                    EXPECT_SUCCESS(s2n_config_set_verification_ca_location(config, S2N_DEFAULT_TEST_CERT_CHAIN, NULL));
-                    EXPECT_NOT_NULL(config->default_certs_by_type.certs[S2N_PKEY_TYPE_RSA]);
-                    EXPECT_SUCCESS(s2n_connection_set_config(client_conn, config));
-                    EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
-
-                    client_conn->actual_protocol_version = S2N_TLS13;
-                    server_conn->actual_protocol_version = S2N_TLS13;
-                    client_conn->secure->cipher_suite = &tls_13_ciphers[i];
-                    server_conn->secure->cipher_suite = &tls_13_ciphers[i];
-
-                    const struct s2n_signature_scheme *chosen_scheme = NULL;
-
-                    if (s2n_is_rsa_pss_signing_supported()) {
-                        /* If RSA PSS signing is supported, then we should always be able to select a default Signature
-                         * Scheme for RSA Certs for TLS 1.3 */
-                        EXPECT_SUCCESS(s2n_tls13_default_sig_scheme(client_conn, &chosen_scheme));
-                        EXPECT_SUCCESS(s2n_tls13_default_sig_scheme(server_conn, &chosen_scheme));
-                    } else {
-                        /* We can't pick a default TLS 1.3 signature scheme when configured with an RSA Cert when we
-                         * do not support RSA PSS signing since RSA PSS signing is required for TLS 1.3 */
-                        EXPECT_FAILURE(s2n_tls13_default_sig_scheme(client_conn, &chosen_scheme));
-                        EXPECT_FAILURE(s2n_tls13_default_sig_scheme(server_conn, &chosen_scheme));
-                    }
-
-                    EXPECT_SUCCESS(s2n_connection_free(client_conn));
-                    EXPECT_SUCCESS(s2n_connection_free(server_conn));
-                    EXPECT_SUCCESS(s2n_config_free(config));
-                    EXPECT_SUCCESS(s2n_cert_chain_and_key_free(default_cert));
-                }
-            }
-
-            /* Same as above test, but with ECDSA Certificates */
-            {
-                struct s2n_cipher_suite tls_13_ciphers[] = {
-                    s2n_tls13_aes_128_gcm_sha256,
-                    s2n_tls13_aes_256_gcm_sha384,
-                    s2n_tls13_chacha20_poly1305_sha256
-                };
-
-                for (size_t i = 0; i < s2n_array_len(tls_13_ciphers); i++) {
-                    struct s2n_config *config = s2n_config_new();
-                    EXPECT_NOT_NULL(config);
-
-                    if (security_policy_selection[policy_index].security_policy->minimum_protocol_version > s2n_get_highest_fully_supported_tls_version()) {
-                        /* We purposefully do not allow users to configure Security Policies with a minimum allowed TLS
-                         * Version of TLS 1.3, if TLS 1.3 algorithms aren't fully supported by the libcrypto we're using */
-                        EXPECT_FAILURE(s2n_config_set_cipher_preferences(config, security_policy_selection[policy_index].version));
-                        EXPECT_SUCCESS(s2n_config_free(config));
-                        continue;
-                    }
-
-                    struct s2n_cert_chain_and_key *default_cert;
-                    EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&default_cert, S2N_DEFAULT_ECDSA_TEST_CERT_CHAIN, S2N_DEFAULT_ECDSA_TEST_PRIVATE_KEY));
-
-                    EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(config, default_cert));
-                    EXPECT_FALSE(config->is_rsa_cert_configured);
-
-                    struct s2n_connection *client_conn = s2n_connection_new(S2N_CLIENT);
-                    struct s2n_connection *server_conn = s2n_connection_new(S2N_SERVER);
-                    EXPECT_NOT_NULL(client_conn);
-                    EXPECT_NOT_NULL(server_conn);
-
-                    EXPECT_SUCCESS(s2n_config_set_cipher_preferences(config, security_policy_selection[policy_index].version));
-                    EXPECT_SUCCESS(s2n_config_set_verification_ca_location(config, S2N_DEFAULT_ECDSA_TEST_CERT_CHAIN, NULL));
-                    EXPECT_NOT_NULL(config->default_certs_by_type.certs[S2N_PKEY_TYPE_ECDSA]);
-                    EXPECT_SUCCESS(s2n_connection_set_config(client_conn, config));
-                    EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
-
-                    client_conn->actual_protocol_version = S2N_TLS13;
-                    server_conn->actual_protocol_version = S2N_TLS13;
-                    client_conn->secure->cipher_suite = &tls_13_ciphers[i];
-                    server_conn->secure->cipher_suite = &tls_13_ciphers[i];
-
-                    const struct s2n_signature_scheme *chosen_scheme = NULL;
-
-                    /* If an ECDSA Certificate is configured, then we should always be able to pick a default Signature
-                     * Scheme (even if RSA PSS is not supported by the libcrypto) */
-                    EXPECT_SUCCESS(s2n_tls13_default_sig_scheme(client_conn, &chosen_scheme));
-                    EXPECT_SUCCESS(s2n_tls13_default_sig_scheme(server_conn, &chosen_scheme));
-
-                    EXPECT_SUCCESS(s2n_connection_free(client_conn));
-                    EXPECT_SUCCESS(s2n_connection_free(server_conn));
-                    EXPECT_SUCCESS(s2n_config_free(config));
-                    EXPECT_SUCCESS(s2n_cert_chain_and_key_free(default_cert));
-                }
-            }
-
             bool has_tls_13_sig_alg = false;
             bool has_rsa_pss = false;
 
@@ -273,7 +151,7 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(0, security_policy->kem_preferences->tls13_kem_group_count);
         EXPECT_NULL(security_policy->kem_preferences->kems);
 
-        /* The "all" security policy contains both TLS 1.2 KEM extension and TLS 1.3 KEM SupportedGroup entries */
+        /* The "all" security policy contains both TLS 1.2 KEM extension and TLS 1.3 KEM SupportedGroup entries*/
         security_policy = NULL;
         EXPECT_SUCCESS(s2n_find_security_policy_from_version("test_all", &security_policy));
         EXPECT_TRUE(s2n_ecc_is_extension_required(security_policy));
@@ -282,15 +160,22 @@ int main(int argc, char **argv)
         EXPECT_NOT_NULL(security_policy->kem_preferences->kems);
         EXPECT_EQUAL(&s2n_kyber_512_r3, security_policy->kem_preferences->kems[0]);
         EXPECT_EQUAL(security_policy->kem_preferences->tls13_kem_groups, pq_kem_groups_r3_2023_06);
-#if defined(S2N_LIBCRYPTO_SUPPORTS_KYBER) && EVP_APIS_SUPPORTED
+        /* All supported kem groups should be in the preference list, but not all of them may be available. */
         EXPECT_EQUAL(6, security_policy->kem_preferences->tls13_kem_group_count);
-#elif defined(S2N_LIBCRYPTO_SUPPORTS_KYBER) && !EVP_APIS_SUPPORTED
-        EXPECT_EQUAL(4, security_policy->kem_preferences->tls13_kem_group_count);
-#elif !defined(S2N_LIBCRYPTO_SUPPORTS_KYBER) && EVP_APIS_SUPPORTED
-        EXPECT_EQUAL(2, security_policy->kem_preferences->tls13_kem_group_count);
-#else
-        EXPECT_EQUAL(1, security_policy->kem_preferences->tls13_kem_group_count);
-#endif
+        uint32_t available_groups = 0;
+        EXPECT_OK(s2n_kem_preferences_groups_available(security_policy->kem_preferences, &available_groups));
+        if (s2n_pq_is_enabled() && s2n_libcrypto_supports_kyber() && s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(6, available_groups);
+        } else if (s2n_pq_is_enabled() && s2n_libcrypto_supports_kyber() && !s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(4, available_groups);
+        } else if (s2n_pq_is_enabled() && !s2n_libcrypto_supports_kyber() && s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(2, available_groups);
+        } else if (s2n_pq_is_enabled()) {
+            EXPECT_EQUAL(1, available_groups);
+        } else {
+            EXPECT_EQUAL(0, available_groups);
+        }
+
         security_policy = NULL;
         EXPECT_SUCCESS(s2n_find_security_policy_from_version("KMS-TLS-1-0-2018-10", &security_policy));
         EXPECT_TRUE(s2n_ecc_is_extension_required(security_policy));
@@ -346,11 +231,14 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(security_policy->kem_preferences->kems, pq_kems_r3_2021_05);
         EXPECT_NOT_NULL(security_policy->kem_preferences->tls13_kem_groups);
         EXPECT_EQUAL(security_policy->kem_preferences->tls13_kem_groups, pq_kem_groups_r3_2021_05);
-#if EVP_APIS_SUPPORTED
-        EXPECT_EQUAL(2, security_policy->kem_preferences->tls13_kem_group_count);
-#else
-        EXPECT_EQUAL(1, security_policy->kem_preferences->tls13_kem_group_count);
-#endif
+        EXPECT_OK(s2n_kem_preferences_groups_available(security_policy->kem_preferences, &available_groups));
+        if (s2n_pq_is_enabled() && s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(2, available_groups);
+        } else if (s2n_pq_is_enabled()) {
+            EXPECT_EQUAL(1, available_groups);
+        } else {
+            EXPECT_EQUAL(0, available_groups);
+        }
 
         security_policy = NULL;
         EXPECT_SUCCESS(s2n_find_security_policy_from_version("PQ-TLS-1-0-2020-12", &security_policy));
@@ -361,11 +249,14 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(security_policy->kem_preferences->kems, pq_kems_r3_2021_05);
         EXPECT_NOT_NULL(security_policy->kem_preferences->tls13_kem_groups);
         EXPECT_EQUAL(security_policy->kem_preferences->tls13_kem_groups, pq_kem_groups_r3_2021_05);
-#if EVP_APIS_SUPPORTED
-        EXPECT_EQUAL(2, security_policy->kem_preferences->tls13_kem_group_count);
-#else
-        EXPECT_EQUAL(1, security_policy->kem_preferences->tls13_kem_group_count);
-#endif
+        EXPECT_OK(s2n_kem_preferences_groups_available(security_policy->kem_preferences, &available_groups));
+        if (s2n_pq_is_enabled() && s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(2, available_groups);
+        } else if (s2n_pq_is_enabled()) {
+            EXPECT_EQUAL(1, available_groups);
+        } else {
+            EXPECT_EQUAL(0, available_groups);
+        }
 
         security_policy = NULL;
         EXPECT_SUCCESS(s2n_find_security_policy_from_version("PQ-TLS-1-1-2021-05-17", &security_policy));
@@ -486,15 +377,20 @@ int main(int argc, char **argv)
         EXPECT_NULL(security_policy->kem_preferences->kems);
         EXPECT_NOT_NULL(security_policy->kem_preferences->tls13_kem_groups);
         EXPECT_EQUAL(security_policy->kem_preferences->tls13_kem_groups, pq_kem_groups_r3_2023_06);
-#if defined(S2N_LIBCRYPTO_SUPPORTS_KYBER) && EVP_APIS_SUPPORTED
+        /* All supported kem groups should be in the preference list, but not all of them may be available. */
         EXPECT_EQUAL(6, security_policy->kem_preferences->tls13_kem_group_count);
-#elif defined(S2N_LIBCRYPTO_SUPPORTS_KYBER) && !EVP_APIS_SUPPORTED
-        EXPECT_EQUAL(4, security_policy->kem_preferences->tls13_kem_group_count);
-#elif !defined(S2N_LIBCRYPTO_SUPPORTS_KYBER) && EVP_APIS_SUPPORTED
-        EXPECT_EQUAL(2, security_policy->kem_preferences->tls13_kem_group_count);
-#else
-        EXPECT_EQUAL(1, security_policy->kem_preferences->tls13_kem_group_count);
-#endif
+        EXPECT_OK(s2n_kem_preferences_groups_available(security_policy->kem_preferences, &available_groups));
+        if (s2n_pq_is_enabled() && s2n_libcrypto_supports_kyber() && s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(6, available_groups);
+        } else if (s2n_pq_is_enabled() && s2n_libcrypto_supports_kyber() && !s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(4, available_groups);
+        } else if (s2n_pq_is_enabled() && !s2n_libcrypto_supports_kyber() && s2n_is_evp_apis_supported()) {
+            EXPECT_EQUAL(2, available_groups);
+        } else if (s2n_pq_is_enabled()) {
+            EXPECT_EQUAL(1, available_groups);
+        } else {
+            EXPECT_EQUAL(0, available_groups);
+        }
 
         security_policy = NULL;
         EXPECT_SUCCESS(s2n_find_security_policy_from_version("20141001", &security_policy));
@@ -564,7 +460,7 @@ int main(int argc, char **argv)
             "20190121",
             "20190122",
             "20201021",
-            "test_all_fips",
+            "20240331",
             "test_all_ecdsa",
             "test_ecdsa_priority",
             "test_all_tls12",
@@ -583,6 +479,7 @@ int main(int argc, char **argv)
             "20190801",
             "20190802",
             "KMS-TLS-1-2-2023-06",
+            "20230317",
             /* CloudFront viewer facing */
             "CloudFront-SSL-v-3",
             "CloudFront-TLS-1-0-2014",
@@ -980,7 +877,7 @@ int main(int argc, char **argv)
                     .kem_count = 0,
                     .kems = NULL,
                     .tls13_kem_group_count = 0,
-                    .tls13_kem_groups = ALL_SUPPORTED_KEM_GROUPS,
+                    .tls13_kem_groups = kem_preferences_all.tls13_kem_groups,
             },
         };
 
@@ -1078,6 +975,48 @@ int main(int argc, char **argv)
                 EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20230317, "default_tls13", rsa_pss_chain_and_key));
                 EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20230317, "20230317", rsa_pss_chain_and_key));
             }
+
+            if (s2n_is_tls13_fully_supported()) {
+                EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20230317,
+                        "test_all_tls13", rsa_chain_and_key));
+                EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20230317,
+                        "test_all_tls13", rsa_pss_chain_and_key));
+                EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20230317,
+                        "test_all_tls13", ecdsa_chain_and_key));
+            }
+        };
+
+        /* 20240331 */
+        {
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "default", rsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "default_tls13", rsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "default_fips", rsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "20230317", rsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "20240331", rsa_chain_and_key));
+
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "default_tls13", ecdsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "default_fips", ecdsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "20230317", ecdsa_chain_and_key));
+            EXPECT_OK(s2n_test_security_policies_compatible(&security_policy_20240331,
+                    "20240331", ecdsa_chain_and_key));
+
+            /* Can't negotiate TLS1.3 */
+            EXPECT_ERROR_WITH_ERRNO(
+                    s2n_test_security_policies_compatible(&security_policy_20240331,
+                            "test_all_tls13", rsa_chain_and_key),
+                    S2N_ERR_CIPHER_NOT_SUPPORTED);
+            EXPECT_ERROR_WITH_ERRNO(
+                    s2n_test_security_policies_compatible(&security_policy_20240331,
+                            "test_all_tls13", ecdsa_chain_and_key),
+                    S2N_ERR_CIPHER_NOT_SUPPORTED);
         };
     };
 
